#!/usr/bin/perl -w
#
# Sucht gleiche oder ähnliche Dateien
# in den angegebenen Verzeichnissen.
#
# Version 1.4
#
# Copyright (C) 2003-2009 Helmar Gerloni
#
# Author: Helmar Gerloni <helmar@gerloni.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# http://www.gnu.org/licenses/gpl.txt
#

# Module (siehe man 3pm Modulname)
use Getopt::Long;
use IO::Handle;
use File::Basename;
use File::Find;
use File::Path;
use File::stat;
use File::Copy;
use File::Path;
use Cwd 'abs_path';
use Digest::MD5;
use Text::Soundex;
use Term::ANSIColor;
use Term::ReadKey;
use Term::ProgressBar;

# set value to be returned for strings without soundex code
$soundex_nocode = 'noSoundexCode';

# Globale Variablen
#  Kommandoname
$cmd = basename($0);
#  Die Optionen werden von GetOptions aus @ARGV entfernt
leseoptionen();
#  Zu durchsuchende Verzeichnisse
@dir = @ARGV;
#  Original-Dateinamen mit Pfad
@files = ();
#  Original-Dateinamen ohne Pfad
@filesbase = ();
#  Strings für die Vergleiche ([bearbeitete] Dateinamen ohne Pfad, MD5-Summen, Dateigröße)
@filescomp = ();
# Hashes für die Ergebnisse
%resultsim = ();
#  Temporäre Variablen
$tmp = '';
@tmp = ();

# Optionen einlesen und Warnungen abfangen
sub leseoptionen {
  local $SIG{__WARN__} = sub { my $warnung; $warnung=$_[0]; chomp($warnung); AufrufFehler("$warnung."); };
  GetOptions ('help' => \$help, 'dir1=s' => \$dir1, 'dir2=s' => \$dir2, 'alg=s' => \$alg, 'verbose' => \$verbose, 'rm2' => \$rm2, 'size' => \$size, 'md5' => \$md5, 'md5db=s' => \$md5db, 'cpto=s' => \$cpto, 'mvto=s' => \$mvto);
}

# Standardausgabe ohne Verzögerung
STDOUT->autoflush(1);

# --help überschreibt alle anderen Optionen
if ($help) {
  Hilfe();
  exit;
}

# Aufruf untersuchen und entsprechende Funktionen aufrufen
if (!$md5db && !$cpto && !$mvto && !$size && !$md5 && $dir1 && $dir2 && !@dir) {
  # Aufruf: [--verbose] [--rm2] [--alg=alg] --dir1=dir1 --dir2=dir2
  pathclean ($dir1,$dir2);
  foreach ($dir1,$dir2) {
    if (!-e) {
      AufrufFehler("Datei oder Verzeichnis $_ nicht gefunden.");
    } elsif (!-d && $rm2) {
     AufrufFehler("Die Option --rm2 ist nur bei Verzeichnissen gültig.");
    }
  }
  # Standardwert für --alg
  if (!$alg) {
    $alg = "sis";
  }
  Aufruf_dir1_dir2();
  if ($alg eq "equ") {
    print "\nDateien mit gleichen Namen:\n";
    Ergebnisausgabe("Folgende Dateien haben den gleichen Namen \"HASHKEY\":");
  } elsif ($alg eq "snx") {
    print "\nDateien mit ähnlichen Namen:\n";
    Ergebnisausgabe("Folgende Namen klingen ähnlich:");
  } else {
    print "\nDateien mit ähnlichen Namen:\n";
    Ergebnisausgabe("Folgende Namen haben die Gemeinsamkeit \"HASHKEY\":");
  }
} elsif (!$md5db && !$cpto && !$mvto && $size && !$md5 && !$alg && $dir1 && $dir2 && !@dir) {
  # Aufruf: [--verbose] [--rm2] --size --dir1=dir1 --dir2=dir2
  pathclean ($dir1,$dir2);
  foreach ($dir1,$dir2) {
    if (!-e) {
      AufrufFehler("Datei oder Verzeichnis $_ nicht gefunden.");
    } elsif (!-d && $rm2) {
     AufrufFehler("Die Option --rm2 ist nur bei Verzeichnissen gültig.");
    }
  }
  Aufruf_dir1_dir2_size();
  print "\nGleich große Dateien:\n";
  Ergebnisausgabe("Folgende Dateien sind HASHKEY Bytes groß:");
} elsif (!$md5db && !$cpto && !$mvto && !$size && $md5 && !$alg && $dir1 && $dir2 && !@dir) {
  # Aufruf: [--verbose] [--rm2] --md5 --dir1=dir1 --dir2=dir2
  pathclean ($dir1,$dir2);
  foreach ($dir1,$dir2) {
    if (!-e) {
      AufrufFehler("Datei oder Verzeichnis $_ nicht gefunden.");
    } elsif (!-d && $rm2) {
     AufrufFehler("Die Option --rm2 ist nur bei Verzeichnissen gültig.");
    }
  }
  # Zuerst Dateigrößen-Vergleich (geht schnell) ...
  Aufruf_dir1_dir2_size();
  # ... dann die MD5-Summen der gleich großen Dateien vergleichen (geht langsam).
  Aufruf_dir1_dir2_md5();
  print "\nGleiche Dateien:\n";
  Ergebnisausgabe("Folgende Dateien sind gleich:");
} elsif (!$rm2 && $md5db && !$size && !$md5 && !$alg && !$dir1 && $dir2 && !@dir) {
  # Aufruf: [--verbose] --md5db=md5db --dir2=dir2 [--cpto=dstdir | --mvto=dstdir]
  if (!-e $md5db) {
    AufrufFehler("Datei $md5db nicht gefunden.");
  } elsif (!-f $md5db) {
    AufrufFehler("$md5db ist keine Datei.");
  }
  pathclean ($dir2);
  if (!-e $dir2) {
    AufrufFehler("Datei oder Verzeichnis $dir2 nicht gefunden.");
  }
  if ($cpto && $mvto) {
    AufrufFehler("Ungültige Optionen-Kombination.");
  } elsif ($cpto) {
    pathclean ($cpto);
    if (!-e $cpto) {
      AufrufFehler("Verzeichnis $cpto nicht gefunden.");
    } elsif (!-d $cpto) {
      AufrufFehler("$cpto ist kein Verzeichnis.");
    }
  } elsif ($mvto) {
    pathclean ($mvto);
    if (!-e $mvto) {
      AufrufFehler("Verzeichnis $mvto nicht gefunden.");
    } elsif (!-d $mvto) {
      AufrufFehler("$mvto ist kein Verzeichnis.");
    }
  }
  print "Suche alle Dateien im Verzeichnis $dir2, die nicht in $md5db verzeichnet sind ...\n";
  Aufruf_md5db_dir2();
  Ergebnisausgabe("Folgende Dateien in $dir2 sind nicht in der Liste $md5db enthalten:");
} elsif (!$rm2 && !$md5db && !$cpto && !$mvto && !$size && !$md5 && !$dir1 && !$dir2 && @dir) {
  # Aufruf: [--verbose] [--alg=alg] dirs
  pathclean (@dir);
  foreach (@dir) {
    # $_ nicht vorhanden
    if (!-e) {
      AufrufFehler("Verzeichnis $_ nicht gefunden.");
    }
    # $_ kein Verzeichnis
    if (!-d) {
      AufrufFehler("$_ ist kein Verzeichnis.");
    }
  }
  # Standardwert für --alg
  if (!$alg) {
    $alg = "sis";
  }
  Aufruf_dirs();
  if ($alg eq "equ") {
    print "\nDateien mit gleichen Namen:\n";
    Ergebnisausgabe("Folgende Dateien haben den gleichen Namen \"HASHKEY\":");
  } elsif ($alg eq "snx") {
    print "\nDateien mit ähnlichen Namen:\n";
    Ergebnisausgabe("Folgende Namen klingen ähnlich:");
  } else {
    print "\nDateien mit ähnlichen Namen:\n";
    Ergebnisausgabe("Folgende Namen haben die Gemeinsamkeit \"HASHKEY\":");
  }
} elsif (!$rm2 && !$md5db && !$cpto && !$mvto && $size && !$md5 && !$alg && !$dir1 && !$dir2 && @dir) {
  # Aufruf: [--verbose] --size dirs
  pathclean (@dir);
  foreach (@dir) {
    # $_ nicht vorhanden
    if (!-e) {
      AufrufFehler("Verzeichnis $_ nicht gefunden.");
    }
    # $_ kein Verzeichnis
    if (!-d) {
      AufrufFehler("$_ ist kein Verzeichnis.");
    }
  }
  Aufruf_dirs_size();
  print "\nGleich große Dateien:\n";
  Ergebnisausgabe("Folgende Dateien sind HASHKEY Bytes groß:");
} elsif (!$rm2 && !$md5db && !$cpto && !$mvto && !$size && $md5 && !$alg && !$dir1 && !$dir2 && @dir) {
  # Aufruf: [--verbose] --md5 dirs
  pathclean (@dir);
  foreach (@dir) {
    # $_ nicht vorhanden
    if (!-e) {
      AufrufFehler("Verzeichnis $_ nicht gefunden.");
    }
    # $_ kein Verzeichnis
    if (!-d) {
      AufrufFehler("$_ ist kein Verzeichnis.");
    }
  }
  # Zuerst Dateigrößen-Vergleich (geht schnell) ...
  Aufruf_dirs_size();
  # ... dann die MD5-Summen der gleich großen Dateien vergleichen (geht langsam).
  Aufruf_dirs_md5();
  print "\nGleiche Dateien:\n";
  Ergebnisausgabe("Folgende Dateien sind gleich:");
} else {
  AufrufFehler("Ungültige Optionen-Kombination.");
}
print "\nEnde.\n";

# Vergleich der Dateinamen in $dir1 und $dir2
sub Aufruf_dir1_dir2 {
  # Datenverteilung in @files, @filesbase, @filescomp:
  # --- 0
  #  |
  #  | alle
  #  | Dateien
  #  | in $dir1
  #  |
  # --- $anfangdir2 - 1
  # --- $anfangdir2
  #  |
  #  | alle
  #  | Dateien
  #  | in $dir2
  #  |
  # --- $#files

  my $anfangdir2 = 0;
  my $fbalken;

  # Alle Dateien einlesen und auswerten
  print "Lese Dateinamen ein ...\n\n";
  # $dir1
  if (-d $dir1) {
    print "Aus Verzeichnis $dir1 ...\n";
    find(\&findAuswertung,$dir1);
  } else {
    print "Aus Datei $dir1 ...\n";
    readAuswertung($dir1);
  }
  if (!@files) {
    LaufzeitFehler("Keine Dateien gefunden.");
  }
  $anfangdir2 = $#files+1;
  print "OK.\n\n$anfangdir2 Dateien aus $dir1 eingelesen.\n\n";
  # $dir2
  if (-d $dir2) {
    print "Aus Verzeichnis $dir2 ...\n";
    find(\&findAuswertung,$dir2);
  } else {
    print "Aus Datei $dir2 ...\n";
    readAuswertung($dir2);
  }
  if ($#files == $anfangdir2-1) {
    LaufzeitFehler("Keine Dateien gefunden.");
  }
  print "OK.\n\n",$#files-$anfangdir2+1," Dateien aus $dir2 eingelesen.\n\n";

  # Dateinamen vergleichen.
  # Verglichen werden in @files alle Einträge von 0 bis $anfangdir2-1 (i)
  # mit allen Einträgen von $anfangdir2 bis $#files (j).
  print "Vergleiche Dateinamen ...\n";
  $fbalken = Term::ProgressBar->new({count=>$anfangdir2});
  for ($i = 0; $i < $anfangdir2; $i++) {
    $fbalken->update($i+1);
    for ($j = $anfangdir2; $j <= $#files; $j++) {

      Vergleich_filescomp();

    }
  }
  print "OK.\n\n";
}

# Vergleich der Größe der Dateien in $dir1 und $dir2
sub Aufruf_dir1_dir2_size {
  # Datenverteilung in @files, @filesbase, @filescomp:
  # --- 0
  #  |
  #  | alle
  #  | Dateien
  #  | in $dir1
  #  |
  # --- $anfangdir2 - 1
  # --- $anfangdir2
  #  |
  #  | alle
  #  | Dateien
  #  | in $dir2
  #  |
  # --- $#files

  my $anfangdir2 = 0;
  my $fbalken;

  # Alle Dateien einlesen und auswerten
  print "Lese Dateigrößen ein ...\n\n";
  # $dir1
  if (-d $dir1) {
    print "Aus Verzeichnis $dir1 ...\n";
    find(\&findAuswertung_size,$dir1);
  } else {
    print "Aus Datei $dir1 ...\n";
    readAuswertung_size($dir1);
  }
  if (!@files) {
    LaufzeitFehler("Keine Dateien gefunden.");
  }
  $anfangdir2 = $#files+1;
  print "OK.\n\n$anfangdir2 Dateien aus $dir1 eingelesen.\n\n";
  # $dir2
  if (-d $dir2) {
    print "Aus Verzeichnis $dir2 ...\n";
    find(\&findAuswertung_size,$dir2);
  } else {
    print "Aus Datei $dir2 ...\n";
    readAuswertung_size($dir2);
  }
  if ($#files == $anfangdir2-1) {
    LaufzeitFehler("Keine Dateien gefunden.");
  }
  print "OK.\n\n",$#files-$anfangdir2+1," Dateien aus $dir2 eingelesen.\n\n";

  # Dateien vergleichen.
  # Verglichen werden in @files alle Einträge von 0 bis $anfangdir2-1 (i)
  # mit allen Einträgen von $anfangdir2 bis $#files (j).
  print "Vergleiche Dateigrößen ...\n";
  $fbalken = Term::ProgressBar->new({count=>$anfangdir2});
  for ($i = 0; $i < $anfangdir2; $i++) {
    $fbalken->update($i+1);
    for ($j = $anfangdir2; $j <= $#files; $j++) {

      Vergleich_filescomp();

    }
  }
  print "OK.\n\n";
}

# Aus dem Ergebnis der Funktion Aufruf_dir1_dir2_size() in %resultsim
# sucht diese Funktion alle Dateien heraus, die den gleichen Inhalt
# (MD5-Summe) haben. Es werden nur die Einträge von dir1 mit denen
# aus dir2 verglichen.
# Das Ergebnis wird wieder nach %resultsim geschrieben.
sub Aufruf_dir1_dir2_md5 {

  my $md5 = new Digest::MD5;
  my $sum = 0;
  my @line = ();
  my $key = '';
  my $fbalken;

  # Lokale Kopie von %resultsim
  my %equalsize = %resultsim;

  # %resultsim löschen, nimmt neues Ergebnis auf
  %resultsim = ();

  # Einträge in %equalsize zählen
  foreach $key (keys %equalsize) {
    # Mit \n getrennte Dateinamen in %equalsize zum Zählen nach @line kopieren
    @line = split(/\n/,$equalsize{$key});
    $sum = $sum+$#line+1;
  }
  # Falls keine Einträge in %equalsize enthalten sind ist hier Schluß.
  if ($sum == 0) {
    print "Keine Dateien mit passender Größe vorhanden, Vergleich der MD5-Summen nicht nötig.\n\n";
    return;
  }
  print "Lese MD5-Summen der verbleibenden $sum Dateien gleicher Größe ein und vergleiche diese ...\n";
  $fbalken = Term::ProgressBar->new({count=>$sum});
  $sum = 0;

  foreach $key (keys %equalsize) {

    # Listen werden neu gefüllt, hier nur
    # mit Dateien gleicher Größe $key
    @files = ();
    @filesbase = ();
    @filescomp = ();

    # Mit \n getrennte Dateinamen in %equalsize nach @files kopieren
    @files = split(/\n/,$equalsize{$key});

    for ($i = 0; $i <= $#files; $i++) {
      $filesbase[$i] = basename($files[$i]);
      $md5->reset();
      # Existenz der Datei ist schon von Aufruf_dirs_size() geprüft worden
      if (-r $files[$i]) {
        open(MD5, $files[$i]) or die color("red"),"Unvorhergesehener Fehler beim Öffnen von $files[$i]: $!\n",color("reset")
;
        $md5->addfile(MD5);
        close MD5;
        $filescomp[$i]=$md5->hexdigest();
        $sum++;
        $fbalken->update($sum);
      } else {
        print "\n";
        LaufzeitFehler("Kann MD5-Summe von $files[$i] nicht lesen.");
      }
    }

    # Dateien vergleichen.
    #  Verglichen wird jeder zweite Eintrag (i) in @files mit dem unmittelbar folgenden (j).
    for ($i = 0; $i <= $#files; $i=$i+2) {
      $j=$i+1;

      Vergleich_filescomp();

    }
  }
  print "OK.\n\n";
}

sub Aufruf_md5db_dir2 {
  # Datenverteilung in @files, @filesbase, @filescomp:
  # --- 0                     ---           ---
  #  |                         |             |
  #  | alle                    |             |
  #  | Dateien                 |        @allmd5sums
  #  | in $md5db               |             |
  #  |                         |             |
  # --- $anfangdir2 - 1        |            ---
  # --- $anfangdir2         @isused
  #  |                         |
  #  | alle                    |
  #  | Dateien                 |
  #  | in $dir2                |
  #  |                         |
  # --- $#files               ---

  my $md5 = new Digest::MD5;
  my $anfangdir2 = 0;
  my $tmp = '';
  my $fbalken;
  my @allmd5sums = ();
  my @isused = ();

  # Alle Dateigrößen einlesen und auswerten
  print "Lese Dateien ein ...\n\n";
  # $md5db
  print "Aus Datei $md5db ...\n";
  open(MD5DB, $md5db) or die color("red"),"Unvorhergesehener Fehler beim Öffnen von $md5db: $!\n",color("reset");
  while (<MD5DB>) {
    # $_ beinhaltet eine Zeile aus MD5DB (Dateigröße MD5-Summe [Dateiname]),
    # Ergebnis wird ans Ende von @files, @filesbase und @filescomp angehängt.
    next if /^\s*(#|$)/;
    chomp;
    $#files++;
    $#filesbase++;
    $#filescomp++;
    $#allmd5sums++;
    ($filescomp[$#filescomp], $allmd5sums[$#allmd5sums], $files[$#files]) = split(' ',$_,3);
    if (!defined($filescomp[$#filescomp]) || !defined($allmd5sums[$#allmd5sums])) {
      print "\n";
      LaufzeitFehler("Fehlerhafter Eintrag in $md5db:\n          $_");
    }
    # Dateiname ist optional
    if ($files[$#files]) {
      $filesbase[$#filesbase] = basename($files[$#files]);
    } else {
      $files[$#files] = '';
      $filesbase[$#filesbase] = '';
    }
    spin();
  }
  close MD5DB;
  if (!@files) {
    LaufzeitFehler("Keine gültigen Einträge gefunden.");
  }
  $anfangdir2 = $#files+1;
  print "OK.\n\n$anfangdir2 Dateien aus $md5db eingelesen.\n\n";
  # $dir2
  if (-d $dir2) {
  print "Aus Verzeichnis $dir2 ...\n";
    find(\&findAuswertung_size,$dir2);
  } else {
    print "Aus Datei $dir2 ...\n";
    readAuswertung_size($dir2);
  }
  if ($#files == $anfangdir2-1) {
    LaufzeitFehler("Keine Dateien gefunden.");
  }
  print "OK.\n\n",$#files-$anfangdir2+1," Dateien aus $dir2 eingelesen.\n\n";

  # Alle Dateien aus $dir2 und alle Einträge aus Datei $md5db in @isused
  # markieren, deren Dateigrößen übereinstimmen und die noch für den
  # MD5-Vergleich benötigt werden
  print "Vergleiche Dateigrößen ...\n";
  $fbalken = Term::ProgressBar->new({count=>$anfangdir2});
  for ($i = 0; $i < $anfangdir2; $i++) {
    $fbalken->update($i+1);
    for ($j = $anfangdir2; $j <= $#files; $j++) {
      if ($filescomp[$i] eq $filescomp[$j]) {
        $isused[$i] = 1;
        $isused[$j] = 1;
      }
    }
  }

  # Nicht benötigte Einträge aus $md5db werden
  # entfernt (kein weiterer Vergleich nötig)
  for ($i = 0; $i < $anfangdir2; $i++) {
    if (!$isused[$i]) {
      splice(@files,$i,1);
      splice(@filesbase,$i,1);
      splice(@filescomp,$i,1);
      splice(@allmd5sums,$i,1);
      splice(@isused,$i,1);
      # Eins zurück, sonst wird Folgeelement ausgelassen
      $i--;
      # Einträge für $dir2 rücken eins vor
      $anfangdir2--;
    }
  }

  # Falls die Dateigröße der Datei aus $dir2 mit keinem Eintrag
  # in Datei $md5db übereinstimmt, wird die Datei ins Ergebnis
  # in %resultsim aufgenommen ...
  print "\n\nBerechne Zwischenergebnis ...\n";
  for ($i = $anfangdir2; $i <= $#files; $i++) {
    spin();
    if (!$isused[$i]) {
      if ($resultsim{md5db_dir2}) {
        $resultsim{md5db_dir2}=$resultsim{md5db_dir2}."$files[$i]\n";
      } else {
        $resultsim{md5db_dir2}="$files[$i]\n";
      }
      # ... und aus @files, @filesbase, @filescomp
      # entfernt (kein weiterer Vergleich nötig)
      splice(@files,$i,1);
      splice(@filesbase,$i,1);
      splice(@filescomp,$i,1);
      splice(@isused,$i,1);
      # Eins zurück, sonst wird Folgeelement ausgelassen
      $i--;
    }
  }
  print "OK.\n\n";

  # Falls keine Dateien aus $dir2 mehr in @files/@filesbase/@filescomp übrig bleiben ist hier Schluß.
  if ($#files == $anfangdir2-1) {
    print "Keine Dateien mit passender Größe vorhanden, Vergleich der MD5-Summen nicht nötig.\n";
    return;
  }

  # Für den Vergleich der MD5-Summen werden die Dateigrößen
  # in @filescomp[0 bis $anfangdir2-1] durch die MD5-Summen
  # ersetzt; ...
  for ($i = 0; $i < $anfangdir2; $i++) {
    $filescomp[$i] = $allmd5sums[$i];
  }
  # ... ebenso die Dateigrößen in @filescomp[$anfangdir2 bis $#@filescomp].
  print "Lese MD5-Summen der verbleibenden ",$#files-$anfangdir2+1," Dateien aus Verzeichnis $dir2 ein ...\n";
  $fbalken = Term::ProgressBar->new({count=>$#files-$anfangdir2+1});
  for ($i = $anfangdir2; $i <= $#files; $i++) {
    $md5->reset();
    open(MD5, $files[$i]) or die color("red"),"Unvorhergesehener Fehler beim Öffnen von $files[$i]: $!\n",color("reset");
    $md5->addfile(MD5);
    close MD5;
    $filescomp[$i]=$md5->hexdigest();
    $fbalken->update($i-$anfangdir2+1);
  }
  print "OK.\n\n";

  # Die MD5-Summen der verbliebenen Dateien in $dir2 werden
  # mit den verbliebenen MD5-Summen aus $md5db verglichen
  print "Vergleiche MD5-Summen ...\n";
  $fbalken = Term::ProgressBar->new({count=>$anfangdir2});
  for ($i = 0; $i < $anfangdir2; $i++) {
    $fbalken->update($i+1);
    for ($j = $anfangdir2; $j <= $#files; $j++) {
      if ($filescomp[$i] eq $filescomp[$j]) {
        # MD5-Summe der Datei in $dir2 ist in $md5db enthalten,
        # $dir2-Eintrag wird aus @files, @filesbase, @filescomp entfernt
        splice(@files,$j,1);
        splice(@filesbase,$j,1);
        splice(@filescomp,$j,1);
        # Eins zurück, sonst wird Folgeelement ausgelassen
        $j--;
      }
    }
  }
  print "OK.\n\n";
  # Die verbleibenden Dateinamen aus $dir2 haben keine Übereinstimmung
  # in $md5db und werden ans Ergebnis in %resultsim angehängt
  for ($i = $anfangdir2; $i <= $#files; $i++) {
    if ($resultsim{md5db_dir2}) {
      $resultsim{md5db_dir2}=$resultsim{md5db_dir2}."$files[$i]\n";
    } else {
      $resultsim{md5db_dir2}="$files[$i]\n";
    }
  }
}

# Vergleich der Dateinamen in @dirs
sub Aufruf_dirs {
  # Datenverteilung in @files, @filesbase, @filescomp:
  # --- 0
  #  |
  #  | alle
  #  | Dateien
  #  | in @dir
  #  |
  # --- $#files

  my $fbalken;

  # Alle Dateien einlesen und auswerten
  print "Lese Dateinamen ein ...\n";
  find(\&findAuswertung,@dir);
  if (!@files) {
    LaufzeitFehler("Keine Dateien gefunden.");
  }
  print "OK.\n\n",$#files+1," Dateinamen aus den Verzeichnissen eingelesen.\n\n";

  # Dateinamen vergleichen.
  #  Verglichen werden alle Einträge in @files mit
  #  allen auf den aktuellen Eintrag (i) folgenden (j).
  print "Vergleiche Dateinamen ...\n";
  $fbalken = Term::ProgressBar->new({count=>$#files});
  for ($i = 0; $i <= $#files; $i++) {
    $fbalken->update($i);
    for ($j = $i+1; $j <= $#files; $j++) {

      Vergleich_filescomp();

    }
  }
  print "OK.\n\n";
}

# Vergleich der Größe der Dateien in @dirs
sub Aufruf_dirs_size {
  # Datenverteilung in @files, @filesbase, @filescomp:
  # --- 0
  #  |
  #  | alle
  #  | Dateien
  #  | in @dir
  #  |
  # --- $#files

  my $fbalken;

  # Alle Dateien einlesen und auswerten
  print "Lese Dateigrößen ein ...\n";
  find(\&findAuswertung_size,@dir);
  if (!@files) {
    LaufzeitFehler("Keine Dateien gefunden.");
  }
  print "OK.\n\n",$#files+1," Dateien aus den Verzeichnissen eingelesen.\n\n";

  # Dateien vergleichen.
  #  Verglichen werden alle Einträge in @files mit
  #  allen auf den aktuellen Eintrag (i) folgenden (j).
  print "Vergleiche Dateigrößen ...\n";
  $fbalken = Term::ProgressBar->new({count=>$#files});
  for ($i = 0; $i <= $#files; $i++) {
    $fbalken->update($i);
    for ($j = $i+1; $j <= $#files; $j++) {

      Vergleich_filescomp();

    }
  }

  print "OK.\n\n";
}

# Aus dem Ergebnis der Funktion Aufruf_dirs_size() in %resultsim
# sucht diese Funktion alle Dateien heraus, die den gleichen Inhalt
# (MD5-Summe) haben. Es werden alle Dateien in %resultsim
# miteinander verglichen.
# Das Ergebnis wird wieder nach %resultsim geschrieben.
sub Aufruf_dirs_md5 {

  my $md5 = new Digest::MD5;
  my $sum = 0;
  my @line = ();
  my $key = '';
  my $tmp = '';
  my @tmp = ();
  my $fbalken;

  # Lokale Kopie von %resultsim
  my %equalsize = %resultsim;

  # %resultsim löschen, nimmt neues Ergebnis auf
  %resultsim = ();

  # Einträge in %equalsize zählen, sortieren und gleiche Einträge entfernen
  foreach $key (keys %equalsize) {
    # Mit \n getrennte Dateinamen in %equalsize nach @line kopieren ...
    @line = split(/\n/,$equalsize{$key});
    # ... sortieren ... (nach @tmp)
    @tmp = sort @line;
    # ... und gleiche Einträge entfernen (zurück in @line, von dort nach %equalsize).
    $tmp = "ungleich $tmp[0]";
    @line = grep($_ ne $tmp && ($tmp = $_, 1), @tmp);
    $equalsize{$key} = join("\n",@line);
    $sum = $sum+$#line+1;
  }
  # Falls keine Einträge in %equalsize enthalten sind ist hier Schluß.
  if ($sum == 0) {
    print "Keine Dateien mit passender Größe vorhanden, Vergleich der MD5-Summen nicht nötig.\n\n";
    return;
  }
  print "Lese MD5-Summen der verbleibenden $sum Dateien gleicher Größe ein und vergleiche diese ...\n";
  $fbalken = Term::ProgressBar->new({count=>$sum});
  $sum = 0;

  foreach $key (keys %equalsize) {

    # Listen werden neu gefüllt, hier nur
    # mit Dateien gleicher Größe $key
    @files = ();
    @filesbase = ();
    @filescomp = ();

    # Mit \n getrennte Dateinamen in %equalsize nach @files kopieren
    @files = split(/\n/,$equalsize{$key});

    for ($i = 0; $i <= $#files; $i++) {
      $filesbase[$i] = basename($files[$i]);
      $md5->reset();
      # Existenz der Datei ist schon von Aufruf_dirs_size() geprüft worden
      if (-r $files[$i]) {
        open(MD5, $files[$i]) or die color("red"),"Unvorhergesehener Fehler beim Öffnen von $files[$i]: $!\n",color("reset");
        $md5->addfile(MD5);
        close MD5;
        $filescomp[$i]=$md5->hexdigest();
        $sum++;
        $fbalken->update($sum);
      } else {
        print "\n";
        LaufzeitFehler("Kann MD5-Summe von $files[$i] nicht lesen.");
      }
    }

    # Dateien vergleichen.
    #  Verglichen werden alle Einträge in @files mit
    #  allen auf den aktuellen Eintrag (i) folgenden (j).
    for ($i = 0; $i <= $#files; $i++) {
      for ($j = $i+1; $j <= $#files; $j++) {

        Vergleich_filescomp();

      }
    }
  }
  print "OK.\n\n";
}

# Vergleich der Strings in $filescomp[$i] mit $filescomp[$j]
# Ergebnis wird nach %resultsim geschrieben
sub Vergleich_filescomp {
  if ($filescomp[$i] eq $filescomp[$j]) {
    if ($resultsim{$filescomp[$i]}) {
      $resultsim{$filescomp[$i]}=$resultsim{$filescomp[$i]}."$files[$i]\n$files[$j]\n";
    } else {
      $resultsim{$filescomp[$i]}="$files[$i]\n$files[$j]\n";
    }
  }

}

# Ausgabe der Ergebnisse in %resultsim
# $_[0]: Meldung bei Ausgabe. Das Wort HASHKEY wird durch den Hashwert ersetzt.
sub Ergebnisausgabe {

  my @results;
  my $nokey = 'true';
  my $text = '';
  my $deldir2 = 'n';
  my $deldir2all = 'n';
  my $dstfile = '';
  my $tmp = '';
  my @tmp = ();

  foreach $key (keys %resultsim) {

    # Mit \n getrennte Dateinamen in %resultsim nach @results kopieren ...
    @results = split(/\n/,$resultsim{$key});
    # ... sortieren ... (nach @tmp)
    @tmp = sort @results;
    # ... und gleiche Einträge entfernen (zurück in @results).
    $tmp = "ungleich $tmp[0]";
    @results = grep($_ ne $tmp && ($tmp = $_, 1), @tmp);
    # Wenn ein Verzeichnis ein Unterverzeichnis des anderen
    # ist kann es zu einzelnen Einträgen kommen; bei --md5db
    # ist auch ein einzelner Eintrag gültig.
    next if ($#results == 0 && !$md5db);

    $nokey = '';
    $text = $_[0];
    $text =~ s/HASHKEY/$key/g;
    print "\n$text\n";

    RESULT: foreach (@results) {
      print "$_\n";
      if ($verbose) {
        fileinfo();
      }
      if ($cpto || $mvto) {
        $dstfile = $_;
        if ($cpto) {
          $dstfile =~ s/^$dir2/$cpto/;
          print "kopiere nach $dstfile ... ";
        } else {
          $dstfile =~ s/^$dir2/$mvto/;
          print "verschiebe nach $dstfile ... ";
        }
        if (!-e dirname($dstfile)) {
          eval {mkpath(dirname($dstfile))};
          if ($@) {
            chomp $@;
            print color("red"),"\nFehler: Kann Verzeichnis ",dirname($dstfile)," nicht anlegen: $@.\n\n",color("reset");
            next RESULT;
          }
        } elsif (!-d dirname($dstfile)) {
          print color("red"),"\nFehler: ",dirname($dstfile)," ist kein Verzeichnis.\n\n",color("reset");
          next RESULT;
        }
        if (-e $dstfile) {
          print color("red"),"\nFehler: kann $dstfile nicht kopieren, Datei existiert bereits.\n\n",color("reset");
          next RESULT;
        }
        if ($cpto) {
          if (copy($_,$dstfile)) {
            print "OK.\n\n";
          } else {
            print color("red"),"\nFehler: Kopiervorgang fehlgeschlagen: $!.\n\n",color("reset");
          }
        } else {
          if (rename($_,$dstfile)) {
            print "OK.\n\n";
          } else {
            print color("red"),"\nFehler: Verschieben fehlgeschlagen: $!.\n\n",color("reset");
          }
        }
      }
    }
    if ($rm2) {
      # Alle Dateien (keine Verzeichnisse, Soft Links usw.), deren
      # Pfad mit $dir2, aber nicht mit $dir1 beginnt, werden
      # zum Löschen angeboten. Dadurch wird aus $dir1 sicher nichts gelöscht,
      # auch wenn $dir1 ein Unterverzeichnis von $dir2 ist.
      # Funktioniert nur zuverlässig, wenn $dir1 und $dir2 Verzeichnisse sind!
      DELDIR2: foreach (@results) {
        if (-f && !-l && $_ =~ /^\Q$dir2\E/ && !($_ =~ /^\Q$dir1\E/)) {
          if ($deldir2all =~ /^(y|j)$/) {
            fdelete($_);
            next DELDIR2;
          }
          ASKAGAIN: print "\nDatei ",color("red"),"$_",color("reset")," löschen? [yjnaq?] ";
          while () {
            ReadMode('cbreak');
            $deldir2 = ReadKey(0);
            ReadMode('normal');
            if ($deldir2 =~ /^(y|j)$/) {
              print "$deldir2\n";
              fdelete($_);
              next DELDIR2;
            } elsif ($deldir2 eq "n") {
              print "$deldir2\n";
              next DELDIR2;
            } elsif ($deldir2 eq "a") {
              print "$deldir2\n";
              print "\nWirklich alle passenden Dateien in ",color("red"),"$dir2",color("reset")," löschen? [yjnq?] ";
              while () {
                ReadMode('cbreak');
                $deldir2all = ReadKey(0);
                ReadMode('normal');
                if ($deldir2all =~ /^(y|j)$/) {
                  print "$deldir2all\n";
                  fdelete($_);
                  next DELDIR2;
                } elsif ($deldir2all eq "n") {
                  print "$deldir2all\n";
                  goto ASKAGAIN;
                } elsif ($deldir2all eq "q") {
                  print "$deldir2all\n";
                  print "\nEnde.\n";
                  exit;
                } elsif ($deldir2 eq "?") {
                  print "$deldir2all\n";
                  print "\n\n y oder j: Alle Dateien in $dir2,\n";
                  print "           die auch in $dir1 gefunden wurden,\n";
                  print "           werden ohne weitere Nachfrage gelöscht.\n";
                  print "    n    : Dateien wird nicht gelöscht.\n";
                  print "    q    : Beendet $cmd.\n";
                }
              }
            } elsif ($deldir2 eq "q") {
              print "$deldir2\n";
              print "\nEnde.\n";
              exit;
            } elsif ($deldir2 eq "?") {
              print "$deldir2\n";
              print "\n\n y oder j: Datei wird gelöscht.\n";
              print "    n    : Datei wird nicht gelöscht.\n";
              print "    a    : Alle Dateien in $dir2,\n";
              print "           die auch in $dir1 gefunden wurden,\n";
              print "           werden nach nochmaliger Bestätigung\n";
              print "           ohne weitere Nachfrage gelöscht.\n";
              print "    q    : Beendet $cmd.\n";
              goto ASKAGAIN;
            }
          }
        }
      }
    }
  }

  if ($nokey) {
    print "\nKeine passenden Dateien gefunden.\n";
  }

}

sub findAuswertung {
  # $File::Find::name beinhaltet beim Aufruf den Dateinamen mit Pfad,
  # $_ beinhaltet beim Aufruf den Dateinamen ohne Pfad,
  # Eintrag wird ans Ende von @files, @filesbase und @filescomp angehängt.
  # @files:     Original-Dateinamen mit Pfad (Dateien, Verzeichnisse, ...)
  # @filesbase: Original-Dateinamen ohne Pfad
  # @filescomp: bearbeitete Dateinamen für die Vergleiche

  if ($_ ne "." && $_ ne "..") {

    $#files++;
    $#filesbase++;
    $#filescomp++;
    $files[$#files] = $File::Find::name;
    $filesbase[$#filesbase] = $_;

    aehnlichLogik();

  }
}

sub findAuswertung_size {
  # $File::Find::name beinhaltet beim Aufruf den Dateinamen mit Pfad,
  # $_ beinhaltet beim Aufruf den Dateinamen ohne Pfad.
  # Funktion find wechselt ins Unterverzeichnis, in welchem sich
  # $_ befindet ($File::Find::dir)!
  # Eintrag wird ans Ende von @files, @filesbase und @filescomp angehängt.
  # @files:     Original-Dateinamen mit Pfad (nur reguläre Dateien)
  # @filesbase: Original-Dateinamen ohne Pfad
  # @filescomp: Dateigrößen für die Vergleiche

  my $status;

  # Reguläre Dateien, keine Links
  if (-f && !-l) {

    $#files++;
    $#filesbase++;
    $#filescomp++;
    $files[$#files] = $File::Find::name;
    $filesbase[$#filesbase] = $_;

    $status = stat($_);
    $filescomp[$#filescomp]=$status->size;
    spin();

  }
}

sub readAuswertung {
  # Liest zeilenweise alle Dateinamen aus der in $_[0]
  # übergebenen Datei aus.
  # Eintrag wird ans Ende von @files, @filesbase und @filescomp angehängt.
  # @files:     Original-Dateinamen mit Pfad (Dateien, Verzeichnisse, ...)
  # @filesbase: Original-Dateinamen ohne Pfad
  # @filescomp: bearbeitete Dateinamen für die Vergleiche

  open(DIRFILE, $_[0]) or die color("red"),"Unvorhergesehener Fehler beim Öffnen von $_[0]: $!\n",color("reset");
  while (<DIRFILE>) {
    # $_ beinhaltet eine Zeile aus DIRFILE (Dateiname mit Pfad),
    # Ergebnis wird ans Ende von @files, @filesbase und @filescomp angehängt.
    next if /^\s*(#|$)/;
    chomp;

    if ($_ ne "." && $_ ne ".." && $_ ne "") {

      pathclean($_);
      $#files++;
      $#filesbase++;
      $#filescomp++;
      $files[$#files] = $_;
      $_ = basename($_);
      $filesbase[$#filesbase] = $_;

      aehnlichLogik();

    }
  }
  close DIRFILE;
}

sub readAuswertung_size {
  # Liest zeilenweise alle Dateinamen aus der in $_[0]
  # übergebenen Datei aus.
  # Eintrag wird ans Ende von @files, @filesbase und @filescomp angehängt.
  # @files:     Original-Dateinamen mit Pfad (Dateien, Verzeichnisse, ...)
  # @filesbase: Original-Dateinamen ohne Pfad
  # @filescomp: Dateigrößen für die Vergleiche

  my $status;

  open(DIRFILE, $_[0]) or die color("red"),"Unvorhergesehener Fehler beim Öffnen von $_[0]: $!\n",color("reset");
  while (<DIRFILE>) {
    # $_ beinhaltet eine Zeile aus DIRFILE (Dateiname mit Pfad),
    # Ergebnis wird ans Ende von @files, @filesbase und @filescomp angehängt.
    next if /^\s*(#|$)/;
    chomp;

    # Reguläre Dateien, keine Links
    if (-f && !-l) {

      pathclean($_);
      $#files++;
      $#filesbase++;
      $#filescomp++;
      $files[$#files] = $_;
      $filesbase[$#filesbase] = basename($_);

      $status = stat($_);
      $filescomp[$#filescomp]=$status->size;
      spin();

    } else {
      print color("red"),"Ignoriere Eintrag $_ ...\n",color("reset");
    }
  }
  close DIRFILE;
}

sub readAuswertung_md5 {
  # Liest zeilenweise alle Dateinamen aus der in $_[0]
  # übergebenen Datei aus.
  # Eintrag wird ans Ende von @files, @filesbase und @filescomp angehängt.
  # @files:     Original-Dateinamen mit Pfad (Dateien, Verzeichnisse, ...)
  # @filesbase: Original-Dateinamen ohne Pfad
  # @filescomp: MD5-Summen für die Vergleiche

  my $md5 = new Digest::MD5;

  open(DIRFILE, $_[0]) or die color("red"),"Unvorhergesehener Fehler beim Öffnen von $_[0]: $!\n",color("reset");
  while (<DIRFILE>) {
    # $_ beinhaltet eine Zeile aus DIRFILE (Dateiname mit Pfad),
    # Ergebnis wird ans Ende von @files, @filesbase und @filescomp angehängt.
    next if /^\s*(#|$)/;
    chomp;

    # Reguläre Dateien, keine Links
    if (-f && ! -l) {

      pathclean($_);
      $#files++;
      $#filesbase++;
      $#filescomp++;
      $files[$#files] = $_;
      $filesbase[$#filesbase] = basename($_);

      $md5->reset();
      open(FILE, $_) or die color("red"),"Unvorhergesehener Fehler beim Öffnen von $_: $!\n",color("reset");
      $md5->addfile(FILE);
      close FILE;
      $filescomp[$#filescomp]=$md5->hexdigest();
      spin();

    }
  }
  close DIRFILE;
}

# "Ähnlich"-Logik für den Dateinamens-Vergleich
# Dateiname (ohne Pfad) in $_ wird in den Vergleichs-String
# konvertiert und das Ergebnis nach $filescomp[$#filescomp] geschrieben.
sub aehnlichLogik {

  my $tmp = '';
  my @tmp = ();

  # Bei --alg=equ werden für die Vergleiche einfach die Dateinamen verwendet ...
  if ($alg eq "equ") {
    $filescomp[$#filescomp] = $_;
    return;
  }
  # ... bei allen anderen --alg=xxx werden die Dateinamen beareitet:

  #  Punkte am Anfang des Dateinamens entfernen
  ($tmp, $_) = $_ =~ /^(\.*)(.*)/;
  #  maximal 4 Zeichen lange Dateiendungen ignorieren
  if ($_ =~ /\./) {
    $_ =~ /^(.+)\.(.*)/;
    # $1: Teil bis zum letzten Punkt
    # $2: Teil nach dem letzten Punkt
    @tmp = split(//, $2);
    if ($#tmp < 4) {
      $_ = $1;
    }
  }

  #  "exotische Buchstaben" abändern (hier gibts sicher noch mehr ...)
  $_ =~ tr/ÀÁÂÃÅàáãåÇçÐÈÉÊËèéêëÌÍÎÏìíîïÑñÒÓÔÕòóôõÙÚÛùúûµÝýÿ/AAAAAaaaaCcDEEEEeeeeIIIIiiiiNnOOOOooooUUUuuuuYyy/;
  #  "normale" deutsche Umlaute und sonstige Buchstabenkombinationen abändern
  $_ =~ s/ß/ss/g;
  $_ =~ s/Ä/Ae/g;
  $_ =~ s/ä/ae/g;
  $_ =~ s/Ö/Oe/g;
  $_ =~ s/ö/oe/g;
  $_ =~ s/Ü/Ue/g;
  $_ =~ s/ü/ue/g;
  $_ =~ s/Æ/AE/g;
  $_ =~ s/æ/ae/g;
  #  Groß/Kleinschreibung ignorieren (Umlaute und sonstige
  #  Sonderbuchstaben werden von lc nicht berücksichtigt)
  $_ = lc();

  if ($alg eq "sim" || $alg eq "sis") {

    # bis hier kann $_ noch nicht leer sein und ist hier
    # der erste brauchbare Dateiname für die Ähnlichkeits-Vergleiche.
    $filescomp[$#filescomp] = $_;

    # Alles ausser Buchstaben und Zahlen löschen
    $_ =~ tr/[a-z][0-9]//cd;
    #  Falls noch Buchstaben vorhanden auch alle Zahlen löschen
    if ($_ =~ /[a-z]/) {
      $_ =~ tr/[a-z]//cd;
    }

    # Bei Option --alg=sis Buchstaben in $_ sortieren
    if ($alg eq "sis") {
      $_ = join('',sort split(//, $_));
    }

    # Wenn $_ auch hier nicht leer dann ist es der neue Dateiname für die Vergleiche.
    if ($_) {
      $filescomp[$#filescomp] = $_;
    } else {
      print color("red"),"Dateiname von $files[$#files] schwer vergleichbar.\n",color("reset");
    }

  } elsif ($alg eq "snx") {

    # Alle Zeichen außer Buchstaben durch _ ersetzen
    $_ =~ tr/[a-z]/_/c;
    # Mehrere _ durch ein _ ersetzen
    $_ =~ s/_+/_/g;
    # _ am Anfang und Ende löschen
    $_ =~ s/^_//;
    $_ =~ s/_$//;
    # Alle übriggebliebenen Worte nach @tmp, dann soundex-Codes nach @tmp und diese sortieren
    @tmp = split(/_/,$_);
    @tmp = soundex @tmp;
    @tmp = sort @tmp;

    $filescomp[$#filescomp] = join('',@tmp);
    if ($filescomp[$#filescomp] eq $soundex_nocode) {
      print color("red"),"Dateiname von $files[$#files] schwer vergleichbar.\n",color("reset");
    }

  } else {

    AufrufFehler("Ungültiger Algorithmus $alg.");

  }

  spin();
}

# Macht übergebene Pfade absolut (erleichtert das
# Erkennen von Unterverzeichnissen)
sub pathclean {

  my $abspath;

  foreach (@_) {
    $abspath = abs_path($_);
    # Unter gewissen Umständen liefert abs_path() kein Ergebnis
    # (wenn nicht existierendes Verzeichnis mit / endet ?)
    if ($abspath) {
      $_ = $abspath;
    }
  }
}

# Löscht die übergebenen Dateien
sub fdelete {
  foreach (@_) {
    if (unlink) {
      print color("red"),"$_",color("reset")," gelöscht.\n";
    } else {
      print color("red"),"Fehler beim Löschen von $_: $!.\n",color("reset");
    }
  }
}

sub fileinfo {

  my $status;

  chomp;
  if (-e) {
    $status = stat($_);
    print "   Größe in Byte: ",$status->size,"  ";
    if (-r) {
      system("file --brief '$_'");
    } else {
      print "(Datei nicht lesbar)\n";
    }
  } elsif (-l) {
    print "   (Soft Link, der ins Leere zeigt)\n";
  } else {
    print "   (Datei existiert nicht)\n";
  }
}

sub linie {
  print "--------------------------------------------------------------------------------\n";
}

# Erzeugt eine kreisende Linie
sub BEGIN {
  my $i = 0;
  sub spin {
    if ($i == 100) {
      print STDERR "-\r";
    } elsif ($i == 200) {
      print STDERR "\\\r";
    } elsif ($i == 300) {
      print STDERR "|\r";
    } elsif ($i == 400) {
      print STDERR "/\r";
      $i = 0;
    }
    $i++;
  }
}

# Fehlerhafter Aufruf
sub AufrufFehler {
  linie();
  print "      ",color("red"),"Fehlerhafter Aufruf:",color("reset"),"\n";
  print "        - $_[0]\n\n";
  print "      ",color("blue"),"$cmd --help",color("reset")," zeigt weitere Informationen an.\n";
  linie();
  exit(1);
}

# Fehler während der Abarbeitung
sub LaufzeitFehler {
  linie();
  print "      ",color("red"),"Fehler:",color("reset"),"\n";
  print "        - $_[0]\n";
  linie();
  exit(1);
}

# Kurzanleitung
sub Hilfe {
  linie();
  print "- Script zum Suchen von gleichen oder ähnlichen Dateien.\n\n";
  print "  ",color("blue"),"$cmd [--verbose] [--rm2] [--alg=",color("green"),"alg",color("blue"),"] --dir1=",color("green"),"dir1 ",color("blue"),"--dir2=",color("green"),"dir2",color("reset"),"\n\n";
  print "    Vergleicht die Dateinamen in ",color("green"),"dir1",color("reset")," mit denen in ",color("green"),"dir2",color("reset")," und gibt je eine Liste\n";
  print "    der Dateien, Verzeichnisse, Links usw. mit gleichen oder ähnlichen\n";
  print "    Dateinamen aus.\n";
  print "    ",color("green"),"dir1",color("reset")," und ",color("green"),"dir2",color("reset")," können Verzeichnisse oder Dateien mit folgendem Inhalt sein:\n\n";
  print "     ",color("red"),"Pfad/zu/Datei1",color("reset"),"\n";
  print "     ",color("red"),"Datei2",color("reset"),"               (z.B. erzeugt mit ",color("blue"),"find . > ",color("green"),"dirx",color("reset"),")\n";
  print "     ",color("red"),"...",color("reset"),"\n\n";
  print "    Mit der Option ",color("blue"),"--alg=",color("green"),"alg",color("reset")," kann der Algorithmus für die Feststellung der\n";
  print "    Ähnlichkeit der Dateinamen ausgewählt werden.\n";
  print "    Gültige Werte für ",color("green"),"alg",color("reset")," sind:\n\n";
  print "     ",color("red"),"equ",color("reset"),": Die Dateinamen sind genau gleich.\n";
  print "     ",color("red"),"sim",color("reset"),": Aus den Dateinamen werden im Wesentlichen alle Sonderzeichen und\n";
  print "          Dateiendungen entfernt, \"exotische\" Buchstaben (Umlaute usw.) durch\n";
  print "          \"normale\" ersetzt (z.B. ä->ae) und das Ergebnis auf Gleichheit\n";
  print "          überprüft.\n";
  print "     ",color("red"),"sis",color("reset"),": Gleich wie ",color("red"),"sim",color("reset"),", nur werden die Buchstaben zusätzlich noch sortiert.\n";
  print "         (Standardeinstellung)\n";
  print "     ",color("red"),"snx",color("reset"),": Es wird festgestellt, ob die Worte in den Dateinamen (auf Englisch)\n";
  print "          gleich ausgesprochen werden (Perl-Modul Text::soundex).\n\n";
  print "  ",color("blue"),"--verbose",color("reset")," zeigt einige zusätzliche Datei-Informationen an.\n\n";
  print "  ",color("blue"),"--rm2",color("reset")," löscht nach Bestätigung die in ",color("green"),"dir2",color("reset")," gefundenen Dateien (keine\n";
  print "     Verzeichnisse, Soft Links usw.). Die Option ist nur gültig, wenn ",color("green"),"dir1",color("reset"),"\n";
print "     und ",color("green"),"dir2",color("reset")," Verzeichnisse sind.\n\n";
  print "  ",color("blue"),"$cmd [--verbose] [--rm2] [--size | --md5] --dir1=",color("green"),"dir1 ",color("blue"),"--dir2=",color("green"),"dir2",color("reset"),"\n\n";
  print "    Vergleicht die Dateigrößen bzw. Dateiinhalte (über MD5-Summen) und\n";
  print "    listet Dateien mit genau gleicher Größe bzw. gleichem Inhalt auf.\n";
  print "    ",color("green"),"dir1",color("reset")," und ",color("green"),"dir2",color("reset")," können Verzeichnisse oder Dateien mit einer Liste von\n";
  print "    Dateinamen sein. Nicht existierende Dateien aus der Liste werden ignoriert.\n\n";
  print "  ",color("blue"),"$cmd [--verbose] [--alg=",color("green"),"alg",color("blue"),"]",color("green")," dir1 ",color("blue"),"[",color("green"),"dir2",color("blue")," ...]",color("reset"),"\n\n";
  print "  ",color("blue"),"$cmd [--verbose] [--size | --md5]",color("green")," dir1 ",color("blue"),"[",color("green"),"dir2",color("blue")," ...]",color("reset"),"\n\n";
  print "    In den Verzeichnissen ",color("green"),"dir1",color("reset")," [",color("green"),"dir2",color("reset")," ...] werden alle Dateien\n";                                                                      print "    miteinander verglichen.\n\n";
  print "  ",color("blue"),"$cmd [--verbose] --md5db=",color("green"),"md5db",color("blue")," --dir2=",color("green"),"dir2",color("blue")," [--cpto=",color("green"),"dstdir",color("blue")," | --mvto=",color("green"),"dstdir",color("blue"),"]",color("reset"),"\n\n";
  print "    Gibt eine Liste der Dateien in ",color("green"),"dir2",color("reset")," aus, die nicht in der Datei\n";
  print "    ",color("green"),"md5db",color("reset")," verzeichnet sind. ",color("green"),"md5db",color("reset")," muß folgendes Format haben:\n\n";
  print "     ",color("red"),"Dateigröße     MD5-Summe    [Dateiname]",color("reset"),"\n";
  print "     ",color("red"),"...",color("reset"),"     (z.B. erzeugt mit\n";
  print "             ",color("blue"),"find . -type f -printf \"%-15s\" -exec md5sum {} \\; > ",color("green"),"md5db",color("reset"),")\n\n";
  print "    ",color("green"),"dir2",color("reset")," kann ein Verzeichnis oder eine Datei mit einer Liste von\n";
  print "    Dateinamen sein.\n";
  print "    Mit ",color("blue"),"--cpto=",color("green"),"dstdir",color("reset")," bzw. ",color("blue"),"--mvto=",color("green"),"dstdir",color("reset")," werden alle in ",color("green"),"dir2",color("reset")," gefundenen Dateien\n";
  print "    unter Beibehaltung der Verzeichnisstruktur nach ",color("green"),"dstdir",color("reset")," kopiert bzw.\n";
  print "    verschoben.\n\n";
  linie();
}

